!!
!! Copyright (c) 2005 The University of Notre Dame. All Rights Reserved.
!!
!! The University of Notre Dame grants you ("Licensee") a
!! non-exclusive, royalty free, license to use, modify and
!! redistribute this software in source and binary code form, provided
!! that the following conditions are met:
!!
!! 1. Redistributions of source code must retain the above copyright
!!    notice, this list of conditions and the following disclaimer.
!!
!! 2. Redistributions in binary form must reproduce the above copyright
!!    notice, this list of conditions and the following disclaimer in the
!!    documentation and/or other materials provided with the
!!    distribution.
!!
!! This software is provided "AS IS," without a warranty of any
!! kind. All express or implied conditions, representations and
!! warranties, including any implied warranty of merchantability,
!! fitness for a particular purpose or non-infringement, are hereby
!! excluded.  The University of Notre Dame and its licensors shall not
!! be liable for any damages suffered by licensee as a result of
!! using, modifying or distributing the software or its
!! derivatives. In no event will the University of Notre Dame or its
!! licensors be liable for any lost revenue, profit or data, or for
!! direct, indirect, special, consequential, incidental or punitive
!! damages, however caused and regardless of the theory of liability,
!! arising out of the use of or inability to use software, even if the
!! University of Notre Dame has been advised of the possibility of
!! such damages.
!!
!! SUPPORT OPEN SCIENCE!  If you use OpenMD or its source code in your
!! research, please cite the appropriate papers when you publish your
!! work.  Good starting points are:
!!                                                                      
!! [1]  Meineke, et al., J. Comp. Chem. 26, 252-271 (2005).             
!! [2]  Fennell & Gezelter, J. Chem. Phys. 124, 234104 (2006).          
!! [3]  Sun, Lin & Gezelter, J. Chem. Phys. 128, 24107 (2008).          
!! [4]  Vardeman & Gezelter, in progress (2009).
!!
!!  Created by Charles F. Vardeman II on 12/6/05.
!!
!!  PURPOSE:
!!
!! @author Charles F. Vardeman II 
!! @version $Id: fForceOptions.F90 1442 2010-05-10 17:28:26Z gezelter $

!! Handles Mixing options for Fortran.

module  fForceOptions 
  use definitions
  implicit none
  PRIVATE
  
#define __FORTRAN90
#include "UseTheForce/fForceOptions.h"
  
  type(ForceOptions), save :: fortranForceOptions
  logical, save :: haveForceOptions = .false.
  real(kind=dp), save, dimension(0:3) :: vdwScale
  real(kind=dp), save, dimension(0:3) :: electrostaticScale
  
  
  public :: ForceOptions
  public :: vdwScale
  public :: electrostaticScale
  public :: getGayBerneMu
  public :: getGayBerneNu
  public :: getEnergyMixingRule
  public :: getDistanceMixingRule
  public :: usesGeometricDistanceMixing
  public :: usesGeometricEnergyMixing
  public :: setForceOptions
  
contains

  subroutine setForceOptions(theseOptions)
    type(ForceOptions),intent(in) :: theseOptions
    fortranForceOptions = theseOptions
    haveForceOptions = .true.
    
    vdwScale(0) = 1.0_dp     ! default is topologically unconnected
    vdwScale(1) = theseOptions%vdw12scale
    vdwScale(2) = theseOptions%vdw13scale
    vdwScale(3) = theseOptions%vdw14scale

    electrostaticScale(0) = 1.0_dp     ! default is topologically unconnected
    electrostaticScale(1) = theseOptions%electrostatic12scale
    electrostaticScale(2) = theseOptions%electrostatic13scale
    electrostaticScale(3) = theseOptions%electrostatic14scale
  end subroutine setForceOptions
    
  function getGayBerneMu() result(thisMu)
    real(kind=dp) :: thisMu
    thisMu = fortranForceOptions%GayBerneMu
  end function getGayBerneMu

  function getGayBerneNu() result(thisNu)
    real(kind=dp) :: thisNu
    thisNu = fortranForceOptions%GayBerneNu
  end function getGayBerneNu
  
  function usesGeometricDistanceMixing() result(doesit)
    logical :: doesit
    doesit = .false.
    if (.not.haveForceOptions) return
    if (fortranForceOptions%DistanceMixingRule == GEOMETRIC_MIXING_RULE) then
       doesit = .true.
    endif
  end function usesGeometricDistanceMixing
    
  function usesGeometricEnergyMixing() result(doesit)
    logical :: doesit
    doesit = .false.
    if (.not.haveForceOptions) return
    if (fortranForceOptions%EnergyMixingRule == GEOMETRIC_MIXING_RULE) then
       doesit = .true.
    endif
  end function usesGeometricEnergyMixing
    
  function getEnergyMixingRule() result(MixingRule)
    integer :: MixingRule
    MixingRule = 0
    if (.not.haveForceOptions) return
    MixingRule = fortranForceOptions%EnergyMixingRule
  end function getEnergyMixingRule
  
  function getDistanceMixingRule() result(MixingRule)
    integer :: MixingRule
    MixingRule = 0
    if (.not.haveForceOptions) return
    MixingRule = fortranForceOptions%DistanceMixingRule
  end function getDistanceMixingRule
  
end module fForceOptions
