!!
!! Copyright (c) 2005 The University of Notre Dame. All Rights Reserved.
!!
!! The University of Notre Dame grants you ("Licensee") a
!! non-exclusive, royalty free, license to use, modify and
!! redistribute this software in source and binary code form, provided
!! that the following conditions are met:
!!
!! 1. Redistributions of source code must retain the above copyright
!!    notice, this list of conditions and the following disclaimer.
!!
!! 2. Redistributions in binary form must reproduce the above copyright
!!    notice, this list of conditions and the following disclaimer in the
!!    documentation and/or other materials provided with the
!!    distribution.
!!
!! This software is provided "AS IS," without a warranty of any
!! kind. All express or implied conditions, representations and
!! warranties, including any implied warranty of merchantability,
!! fitness for a particular purpose or non-infringement, are hereby
!! excluded.  The University of Notre Dame and its licensors shall not
!! be liable for any damages suffered by licensee as a result of
!! using, modifying or distributing the software or its
!! derivatives. In no event will the University of Notre Dame or its
!! licensors be liable for any lost revenue, profit or data, or for
!! direct, indirect, special, consequential, incidental or punitive
!! damages, however caused and regardless of the theory of liability,
!! arising out of the use of or inability to use software, even if the
!! University of Notre Dame has been advised of the possibility of
!! such damages.
!!
!! SUPPORT OPEN SCIENCE!  If you use OpenMD or its source code in your
!! research, please cite the appropriate papers when you publish your
!! work.  Good starting points are:
!!                                                                      
!! [1]  Meineke, et al., J. Comp. Chem. 26, 252-271 (2005).             
!! [2]  Fennell & Gezelter, J. Chem. Phys. 124, 234104 (2006).          
!! [3]  Sun, Lin & Gezelter, J. Chem. Phys. 128, 24107 (2008).          
!! [4]  Vardeman & Gezelter, in progress (2009).
!!

!! Machine and compiler dependent definitions
!! Charles F. Vardeman II 2/26/02
!! PUBLIC VARIABLES
!! DEFAULT_INPUT    default standard input
!! DEFAULT_OUTPUT   default standard output
!! DEFAULT_ERROR    default standard error
!! SP               single precision type
!! DP               double precision type
!! MAX_UNITS        system dependend maximum number of open units

module definitions
  IMPLICIT NONE
  PUBLIC

  !! Machine dependent input and output (fortran 2000 will fix this standard)
  INTEGER, PARAMETER :: DEFAULT_INPUT  = 5
  INTEGER, PARAMETER :: DEFAULT_OUTPUT = 6
  INTEGER, PARAMETER :: DEFAULT_ERROR  = 0

  !! Various precision parameters

  INTEGER, PARAMETER :: SP = selected_real_kind(4)
#ifdef SINGLE_PRECISION
  INTEGER, PARAMETER :: DP = selected_real_kind(4)
#else
  INTEGER, PARAMETER :: DP = selected_real_kind(8)
#endif

  !! Maximum number of fortran streams...
  INTEGER, PARAMETER :: MAX_UNITS = 100

  !! number of dimensions in simulation
  INTEGER, PARAMETER :: ndim = 3
  !! Default Size parameter of nlists
  INTEGER, PARAMETER :: nlistPrefactor = 80

end module definitions
